#! /bin/bash
# -----------------------------------------------------------------------------
# f2cad: Convert Fortran to C++ AD Types: Copyright (C) 2005-10 Bradley M. Bell
# Authors:  Bradley M. Bell    bradbell at washington dot edu
# License:  GNU Public License Version 2 or higher
# -----------------------------------------------------------------------------
# Bash script for building the f2cad distribution.
#
# Default valeus used for arguments to configure during this script.
# These defaults are development system dependent and may need to be changed. 
ADOLC_DIR="$HOME/prefix/adolc"
CPPAD_DIR="$HOME/prefix/cppad"
FADBAD_DIR="$HOME/prefix/fadbad"
CXX_FLAGS="-g -Wall -ansi -pedantic-errors -std=c++98"
# ---------------------------------------------------------------------------
F2C_DIR=`which f2c | sed -e 's|/bin/f2c||'`
# ---------------------------------------------------------------------------
missing="no"
for dir in f2c.p cpp f.omh p.omh f2cad
do
	if [ ! -d $dir ]
	then
		missing="yes"
	fi
done
if [ "$missing" = "yes" ] 
then
	echo "./build_sources.sh"
	if ! ./build_sources.sh $F2C_DIR
	then
		echo "build_sources.sh failed."
		exit 1
	fi
fi
#
# get version currently in configure.ac file
# (in a way that works when version is not a date)
configure_ac_version=`grep "^ *AC_INIT(" configure.ac | \
	sed -e 's/[^,]*, *\([^ ,]*\).*/\1/'`
#
# version
if [ "$1" = "version" ] || [ "$1" = "all" ]
then
	#
	# Today's date in yy-mm-dd decimal digit format with leading zeros
	yyyy_mm_dd=`date +%F`
	yyyymmdd=`echo $yyyy_mm_dd | sed -e 's|-||g'`
	#
	sed < configure.ac > configure.ac.$$\
		-e "s/(f2cad, [.0-9]* *,/(f2cad, $yyyymmdd,/"
	diff configure.ac    configure.ac.$$
	mv   configure.ac.$$ configure.ac
	#
	# configure file is out of date so remove it
	if [ -e configure ]
	then
		rm configure
	fi
	#
	if [ "$1" != "all" ]
	then
		exit 0
	fi
	configure_ac_version="$yyyymmdd"
fi
version="$configure_ac_version"
#
# autoreconf
if [ "$1" = "autoreconf" ] || [ "$1" = "all" ]
then
	if [ -e configure ]
	then
		if ! rm configure
		then
			echo "build.sh: cannot remove old ./configure file"
			exit 1
		fi
	fi
	echo "autoreconf --force --install"
	autoreconf --force --install
	if [ "$1" != "all" ]
	then
		exit 0
	fi
fi
#
# configure
#
if [ "$1" = "configure" ] || [ "$1" = "all" ]
then
	echo "configure \\"
	echo "  F2C_DIR=$F2C_DIR \\"
	echo "  ADOLC_DIR=$ADOLC_DIR \\"
	echo "  FADBAD_DIR=$FADBAD_DIR \\"
	echo "  CPPAD_DIR=$CPPAD_DIR \\"
	echo "  CXX_FLAGS=$CXX_FLAGS"
	if ! ./configure \
		F2C_DIR="$F2C_DIR" \
		ADOLC_DIR="$ADOLC_DIR" \
		FADBAD_DIR="$FADBAD_DIR" \
		CPPAD_DIR="$CPPAD_DIR" \
		CXX_FLAGS="$CXX_FLAGS"
	then
		echo "build.sh: error during configure command."
		exit 1
	fi
	#
	if [ "$1" != "all" ]
	then
		exit 0
	fi
fi
#
# make
#
if [ "$1" = "make" ] || [ "$1" = "all" ]
then
	echo "make"
	if ! make
	then
		echo "build.sh make failed."
		exit 1
	fi
	#
	if [ "$1" != "all" ]
	then
		exit 0
	fi
fi
#
# test
#
if [ "$1" = "test" ] || [ "$1" = "all" ]
then
	echo "make test"
	if ! make test
	then
		echo "build.sh: make test failed."
		exit 1
	fi
	#
	if [ "$1" != "all" ]
	then
		exit 0
	fi
fi
#
# doc
if [ "$1" = "doc" ] || [ "$1" = "all" ]
then
	echo "run_omhelp.sh"
	if ! ./run_omhelp.sh 
	then
		echo "build.sh error during ./run_omhelp.sh command."
		exit 1
	fi
	#
	if [ "$1" != "all" ]
	then
		exit 0
	fi
fi
#
# dist
#
if [ "$1" = "dist" ] || [ "$1" = "all" ]
then
	#
	if [ -e f2cad-$version ]
	then
		echo "rm -f -r f2cad-$version"
		if ! rm -f -r f2cad-$version
		then
			echo "build.sh: cannot remove old f2cad-$version"
			exit 1
		fi
	fi
	#
	echo "make dist"
	if ! make dist
	then
		echo "build.sh: error during make dist command."
		exit 1
	fi
	#
	exit 0
fi
if [ "$1" = "" ]
then
	echo "usage: build.sh option (where valid options are listed below)" 
else
	echo "$1 is not a valid option (valid options are listed below)"
fi
echo "option"
echo "------"
echo "version     update configure.ac and f2cad.omh version number"
echo "autoreconf  run autoreconf"
echo "configure   run configure script to create make files"
echo "make        compile the library"
echo "test        compile and run the tests"
echo "doc         build all the documentation"
echo "dist        create the distribution files f2cad.*.tar.gz"
echo
echo "build.sh all: execute all the options in the order above"
exit 1
